@@ -15,6 +15,7 @@ mod preview;
 mod util;
 
 use std::collections::HashMap;
+use structopt::StructOpt;
 
 use lsp_server::{Connection, Message, Request, RequestId, Response};
 use lsp_types::notification::{DidChangeTextDocument, DidOpenTextDocument, Notification};
@@ -40,6 +41,24 @@ type Error = Box<dyn std::error::Error>;
 
 const SHOW_PREVIEW_COMMAND: &str = "showPreview";
 
+#[derive(StructOpt, Clone)]
+struct Cli {
+    #[structopt(
+        short = "I",
+        name = "Add include paths for the import statements",
+        number_of_values = 1
+    )]
+    include_paths: Vec<std::path::PathBuf>,
+
+    /// The style name for the preview ('native' or 'ugly')
+    #[structopt(long, name = "style name", default_value)]
+    style: String,
+
+    /// The backend used for the preview ('GL' or 'Qt')
+    #[structopt(long, name = "backend", default_value)]
+    backend: String,
+}
+
 pub struct DocumentCache<'a> {
     documents: TypeLoader<'a>,
     newline_offsets: HashMap<Url, Vec<u32>>,
@@ -95,24 +114,38 @@ impl<'a> DocumentCache<'a> {
 }
 
 fn main() {
-    std::thread::spawn(|| {
+    let args: Cli = Cli::from_args();
+    if !args.backend.is_empty() {
+        std::env::set_var("SIXTYFPS_BACKEND", &args.backend);
+    }
+
+    let lsp_thread = std::thread::spawn(|| {
+        /// Make sure we quit the event loop even if we panic
+        struct QuitEventLoop;
+        impl Drop for QuitEventLoop {
+            fn drop(&mut self) {
+                preview::quit_ui_event_loop();
+            }
+        }
+        let _quit_ui_loop = QuitEventLoop;
+
         match run_lsp_server() {
             Ok(_) => {}
             Err(error) => {
                 eprintln!("Error running LSP server: {}", error);
             }
         }
-        preview::quit_ui_event_loop();
     });
 
     preview::start_ui_event_loop();
+    lsp_thread.join().unwrap();
 }
 
 fn run_lsp_server() -> Result<(), Error> {
     let (connection, io_threads) = Connection::stdio();
     let capabilities = ServerCapabilities {
         completion_provider: Some(CompletionOptions {
-            resolve_provider: Some(true),
+            resolve_provider: None,
             trigger_characters: Some(vec![".".to_owned()]),
             work_done_progress_options: WorkDoneProgressOptions::default(),
         }),
@@ -137,11 +170,13 @@ fn run_lsp_server() -> Result<(), Error> {
 }
 
 fn main_loop(connection: &Connection, params: serde_json::Value) -> Result<(), Error> {
+    let cli_args: Cli = Cli::from_args();
     let params: InitializeParams = serde_json::from_value(params).unwrap();
-    let mut compiler_config = sixtyfps_compilerlib::CompilerConfiguration::new(
-        sixtyfps_compilerlib::generator::OutputFormat::Interpreter,
-    );
-    compiler_config.style = Some("ugly".into());
+    let mut compiler_config =
+        CompilerConfiguration::new(sixtyfps_compilerlib::generator::OutputFormat::Interpreter);
+    compiler_config.style =
+        Some(if cli_args.style.is_empty() { "ugly".into() } else { cli_args.style });
+    compiler_config.include_paths = cli_args.include_paths;
 
     let mut document_cache = DocumentCache::new(&compiler_config);
     for msg in &connection.receiver {
@@ -310,7 +345,7 @@ fn handle_notification(
 fn show_preview_command(
     params: &[serde_json::Value],
     connection: &Connection,
-    document_cache: &DocumentCache,
+    _document_cache: &DocumentCache,
 ) -> Result<(), Error> {
     let e = || -> Error { "InvalidParameter".into() };
     let path = if let serde_json::Value::String(s) = params.get(0).ok_or_else(e)? {
@@ -320,19 +355,9 @@ fn show_preview_command(
     };
     let path_canon = dunce::canonicalize(&path).unwrap_or_else(|_| path.to_owned());
     let component = params.get(1).and_then(|v| v.as_str()).map(|v| v.to_string());
-    let is_window = component
-        .as_ref()
-        .and_then(|c| {
-            let mut ty = document_cache.documents.get_document(&path)?.local_registry.lookup(&c);
-            while let Type::Component(c) = ty {
-                ty = c.root_element.borrow().base_type.clone();
-            }
-            Some(matches!(ty, Type::Builtin(b) if b.name == "Window"))
-        })
-        .unwrap_or(false);
     preview::load_preview(
         connection.sender.clone(),
-        preview::PreviewComponent { path: path_canon.into(), component, is_window },
+        preview::PreviewComponent { path: path_canon.into(), component },
         preview::PostLoadBehavior::ShowAfterLoad,
     );
     Ok(())
@@ -363,16 +388,11 @@ fn maybe_goto_preview(
         if let Some(component) = syntax_nodes::Component::new(node.clone()) {
             let component_name =
                 sixtyfps_compilerlib::parser::identifier_text(&component.DeclaredIdentifier())?;
-            let is_window = component
-                .Element()
-                .QualifiedName()
-                .map_or(false, |q| q.text().to_string().trim() == "Window");
             preview::load_preview(
                 sender,
                 preview::PreviewComponent {
                     path: token.source_file.path().into(),
                     component: Some(component_name),
-                    is_window,
                 },
                 preview::PostLoadBehavior::ShowAfterLoad,
             );
@@ -411,7 +431,7 @@ fn reload_document(
             continue;
         }
         let uri = Url::from_file_path(d.source_file().unwrap()).unwrap();
-        lsp_diags.entry(uri).or_default().push(to_lsp_diag(&d));
+        lsp_diags.entry(uri).or_default().push(util::to_lsp_diag(&d));
     }
 
     for (uri, diagnostics) in lsp_diags {
@@ -424,32 +444,6 @@ fn reload_document(
     Ok(())
 }
 
-fn to_lsp_diag_level(
-    level: sixtyfps_compilerlib::diagnostics::DiagnosticLevel,
-) -> lsp_types::DiagnosticSeverity {
-    match level {
-        sixtyfps_interpreter::DiagnosticLevel::Error => lsp_types::DiagnosticSeverity::Error,
-        sixtyfps_interpreter::DiagnosticLevel::Warning => lsp_types::DiagnosticSeverity::Warning,
-    }
-}
-
-fn to_lsp_diag(d: &sixtyfps_compilerlib::diagnostics::Diagnostic) -> lsp_types::Diagnostic {
-    lsp_types::Diagnostic::new(
-        to_range(d.line_column()),
-        Some(to_lsp_diag_level(d.level())),
-        None,
-        None,
-        d.message().to_owned(),
-        None,
-        None,
-    )
-}
-
-fn to_range(span: (usize, usize)) -> Range {
-    let pos = Position::new((span.0 as u32).saturating_sub(1), (span.1 as u32).saturating_sub(1));
-    Range::new(pos, pos)
-}
-
 /// return the token, and the offset within the file
 fn token_descr(
     document_cache: &mut DocumentCache,
